package com.argotec.siar.scraper;

import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.Province;
import com.argotec.siar.model.Station;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Slf4j
public class ScraperStationInfo implements ScraperDataSource<Station> {

    private final static String BASE_URL = "http://eportal.mapama.gob.es/websiar/";

    @Override
    public List<Station> scrapeAll() {
        log.info("Fetching information for all stations");

        String url = BASE_URL.concat("SeleccionParametrosMap.aspx?dst=1");

        try {
            Document doc = Jsoup.connect(url).get();
            Elements elements = doc.select("#ContenedorMapaGrande > img");
            int numberOfStations = elements.size();
            log.info("There are {} stations", numberOfStations);
            AtomicInteger i = new AtomicInteger();
            return elements.stream()
                    .skip(1)
                    .map(element -> {
                        log.info("Element {}/{}", i.getAndIncrement(), numberOfStations);
                        return fetchStationInfo(element.id(), element.attr("title"));
                    })
                    .collect(Collectors.toList());
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Station scrapeOne(String stationName) {
        String url = BASE_URL.concat("SeleccionParametrosMap.aspx?dst=1");
        log.info("Finding station {}", stationName);
        Station station = null;

        try {
            Document doc = Jsoup.connect(url).get();
            Element element = doc.selectFirst(String.format("img[title='%s']", stationName));

            if (element != null) {
                log.info("Station {} found", stationName);
                station = fetchStationInfo(element.id(), element.attr("title"));
            } else {
                log.info("Station {} NOT found", stationName);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return station;
    }

    public Station fetchStationInfo(String stationId, String stationName) {
        log.info("Fetching from {} ({})", stationName, stationId);
        String url = BASE_URL.concat("SeleccionParametrosMap.aspx/DevolverCadenasBocadillo");
        RestTemplate restTemplate = new RestTemplate();
        Map<String, String> payload = new HashMap<>();
        payload.put("NombreCortoEstacion", stationId);

        ResponseEntity<RequestPayload> response = restTemplate.postForEntity(url, payload, RequestPayload.class);

        RequestPayload responseBody = response.getBody();
        Station station = new Station();
        station = responseBody.updateStation(station);
        station = updateStationInfoFromWebsite(responseBody.getD().get(7), station);
        return station;
    }

    private Station updateStationInfoFromWebsite(String stationReferencedLink, Station station) {
        String url = BASE_URL.concat(stationReferencedLink);

        try {
            Document doc = Jsoup.connect(url).get();

            station.setProperty(getTextFrom(doc, "#lblPropiedad"));
            station.setParage(getTextFrom(doc, "#lblParaje"));
            station.setBasin(getTextFrom(doc, "#lblCuenca"));
            station.setUtmx(getIntegerFrom(doc, "#lblUTMX"));
            station.setUtmy(getIntegerFrom(doc, "#lblUTMY"));
            station.setTimeZone(getIntegerFrom(doc, "#lblHuso"));
            station.setAltitude(getIntegerFrom(doc, "#lblAltitud"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        return station;
    }

    private String getTextFrom(Document doc, String cssQuery) {
        String text = doc.selectFirst(cssQuery).text();
        try {
            return text.split(": ")[1];
        } catch (ArrayIndexOutOfBoundsException ex) {
            log.warn("Unable to split {}", text);
            return text;
        }
    }

    private Integer getIntegerFrom(Document doc, String cssQuery) {
        return Integer.parseInt(getTextFrom(doc, cssQuery));
    }

    @Getter
    @Setter
    @ToString
    private static class RequestPayload {
        private List<String> d;

        public Station updateStation(Station station) {
            station.setName(d.get(0).split("\\(")[0].trim());
            station.setProvince(new Province(d.get(1), null));
            station.setMunicipality(d.get(2));
            station.setState("Activa".equalsIgnoreCase(d.get(3)));
            String instalationDate = d.get(4);
            station.setLastMeasurementDate(d.get(5));
            String provinceId = d.get(8);
            String stationId = d.get(9);
            String stationUniqId = d.get(10);
            return station;
        }
    }
}
