package com.argotec.siar.service.impl;

import com.argotec.siar.model.Crop;
import com.argotec.siar.model.IrrigationCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.model.Station;
import com.argotec.siar.repository.CropRepository;
import com.argotec.siar.repository.ProvinceRepository;
import com.argotec.siar.repository.StationRepository;
import com.argotec.siar.scraper.ScraperCrops;
import com.argotec.siar.scraper.ScraperStationInfo;
import com.argotec.siar.scraper.ScraperStations;
import com.argotec.siar.service.StationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static com.argotec.siar.util.FormatUtil.stringsAreEqualAfterNormalization;

@Service
@Slf4j
public class StationServiceImpl implements StationService {

    private final StationRepository stationRepository;
    private final CropRepository cropRepository;
    private final ProvinceRepository provinceRepository;

    public StationServiceImpl(StationRepository stationRepository, CropRepository cropRepository, ProvinceRepository provinceRepository) {
        this.stationRepository = stationRepository;
        this.cropRepository = cropRepository;
        this.provinceRepository = provinceRepository;
    }

    @Override
    public Station getByName(String stationName) {
        log.info("Getting station '{}'", stationName);

        Station station = stationRepository
                .findAll()
                .stream()
                .filter(s -> stringsAreEqualAfterNormalization(s.getName(), stationName))
                .findFirst()
                .orElseGet(() -> {
                    log.info("Station '{}' not found, fetching using web scraper", stationName);
                    Station _station = new ScraperStationInfo().scrapeOne(stationName);
                    if (_station != null) {
                        Province province = provinceRepository.findByName(_station.getProvince().getName());
                        _station.setProvince(province);
                        stationRepository.save(_station);
                    }
                    return _station;
                });

        if (station != null) {
            return station;
        } else {
            throw new RuntimeException(String.format("Station '%s' not found", stationName));
        }
    }

    @Override
    public List<Station> getAllStations(boolean ws) {
        log.info("Getting all stations, forcing to use web scraper? {}", ws);
        List<Station> list = stationRepository.findAll();

        if ((ws || list.isEmpty())) {
            log.info("Fetching all stations using web scraper");
            list = new ScraperStationInfo().scrapeAll();
            saveStationList(list);
        }

        return list;
    }

    @Override
    public List<Station> getAllStations(Province province, boolean ws) {
        log.info("Getting all stations belonging to {} province, forcing to use web scraper? {}", province.getName(), ws);
        List<Station> list = stationRepository.findAllByProvince(province);

        if (ws || list.isEmpty()) {
            log.info("Fetching stations using web scraper");
            List<Crop> existingCrops = cropRepository.findAll();
            list = getScrapper(existingCrops, province).scrapeAll();
            cropRepository.saveAll(existingCrops);
            saveStationList(list);
        }

        return list;
    }

    @Override
    public List<Station> getAllStations(IrrigationCommunity irrigationCommunity, boolean ws) {
        log.info("Getting all stations belonging to {} irrigation community, forcing to use web scraper? {}", irrigationCommunity.getName(), ws);
        List<Station> list = stationRepository.findAllByIrrigationCommunity(irrigationCommunity);

        if (ws || list.isEmpty()) {
            log.info("Fetching stations using web scraper");
            List<Crop> existingCrops = cropRepository.findAll();
            list = getScrapper(existingCrops, irrigationCommunity).scrapeAll();
            cropRepository.saveAll(existingCrops);
            saveStationList(list);
        }

        return list;
    }

    @Override
    public Set<Crop> getCropsByStation(Station station, boolean ws) {
        log.info("Getting all crops belonging to {} station, forcing to use web scraper? {}", station.getName(), ws);
        Set<Crop> stationCrops = station.getCrops();

        if (ws || stationCrops.isEmpty()) {
            log.info("Fetching crops using web scraper");
            List<Crop> existingCrops = cropRepository.findAll();
            stationCrops = new HashSet<>(new ScraperCrops(existingCrops, station).scrapeAll());

            cropRepository.saveAll(existingCrops);

            station.setCrops(stationCrops);
            stationRepository.save(station);
        }

        return stationCrops;
    }

    private ScraperStations getScrapper(List<Crop> existingCrops, Province province) {
        return new ScraperStations(existingCrops, province.getAutonomousCommunity().getName(), province);
    }

    private ScraperStations getScrapper(List<Crop> existingCrops, IrrigationCommunity irrigationCommunity) {
        return new ScraperStations(existingCrops, irrigationCommunity.getAutonomousCommunity().getName(), irrigationCommunity);
    }

    private void saveStationList(List<Station> stations) {
        stations.stream()
                .filter(s -> stationRepository.findByName(s.getName()) == null)
                .peek(s -> log.info("Saving {} station in the repository", s.getName()))
                .forEach(stationRepository::save);
    }
}
