package com.argotec.restapi;

import com.argotec.scraper.Scraper;
import com.argotec.siar.util.ScrapingUrl;
import org.junit.Test;
import org.openqa.selenium.WebDriverException;

import java.time.LocalDate;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.siar.scraper.ScraperCommunities.selectAuCommunityAction;
import static com.argotec.siar.scraper.ScraperIrrigationCommunities.selectIrrigationCommunityAction;
import static com.argotec.siar.scraper.ScraperIrrigationCommunities.selectIrrigationCommunityRadioButton;
import static com.argotec.siar.scraper.ScraperStations.addStation;
import static com.argotec.siar.scraper.SiarScraperUtil.setStartDate;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

public class UserStoryTests extends IntegrationTestScaffolding {

    @Test
    @UserStory("US-001")
    public void shouldGetWelcomeMessageFromHomeController() throws Exception {
        mockMvc.perform(get("/v1/index"))
                .andExpect(content().string("Welcome to your API REST to get meteorological data"));
    }

    @Test
    @UserStory("US-001")
    public void shouldReturn404WhenDoAGetToAnInvalidResourcePath() throws Exception {
        mockMvc.perform(get("/v1/home"))
                .andExpect(status().isNotFound());
    }

    @Test
    @UserStory("US-001")
    public void shouldReturn200WhenDoAGetToAValidResourcePath() throws Exception {
        mockMvc.perform(get("/v1/index"))
                .andExpect(status().isOk());
    }

    @Test
    @UserStory("US-002")
    public void shouldScrapeDataFromAWebsite() throws Exception {
        mockMvc.perform(get("/v1/irrigation-communities/Aleje?communityName=Castilla y Leon&ws=true"))
                .andExpect(jsonPath("$.name", equalTo("Aleje")))
                .andExpect(jsonPath("$.autonomousCommunity.name", equalTo("Castilla y León")));
    }

    @Test
    @UserStory({"US-003", "US-004", "US-018"})
    public void shouldScraperConnectToAValidWebsite() {
        try (Scraper scraper = source("https://www.uva.es")) {
            assertThat(scraper.getTitle(), equalTo("Universidad de Valladolid - SSO"));
        }
    }

    @Test
    @UserStory({"US-003", "US-004", "US-018"})
    public void shouldReturnAnErrorWhenScraperConnectToAnInvalidWebsite() {
        try (Scraper scraper = source("https://www.webSite.that.doesNotExist")) {
        } catch (WebDriverException ex) {
            assertThat(ex.getMessage(), startsWith("Reached error page: about:neterror?e=dnsNotFound&u=https%3A//www.website.that.doesnotexist"));
        }
    }

    @Test
    @UserStory({"US-006", "US-011", "US-019"})
    public void shouldFilterUsingAnOptionOfASelectElement() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectIrrigationCommunityRadioButton())
                    .doAction(selectIrrigationCommunityAction("Alamo Gordo"));
        }
    }

    @Test
    @UserStory({"US-006", "US-011", "US-019"})
    public void shouldThrowAnExceptionWhenTryingToFilterUsingAnInvalidOptionOfASelectElement() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectIrrigationCommunityRadioButton())
                    .doAction(selectIrrigationCommunityAction("aOptionDoesNotExist"));
        } catch (IllegalArgumentException ex) {
            assertThat(ex.getMessage(), equalTo("Error: Incorrectly introduced irrigation community"));
        }
    }

    @Test
    @UserStory({"US-005", "US-007", "US-008", "US-009", "US-020"})
    public void shouldSelectAnOptionOfASelectElement() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectAuCommunityAction("Castilla y León"));
        }
    }

    @Test
    @UserStory({"US-005", "US-007", "US-008", "US-009", "US-020"})
    public void shouldThrowAnExceptionWhenTryingToSelectAnInvalidOptionOfASelectElement() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectAuCommunityAction("aCommunityThatDoesNotExist"));
        } catch (IllegalArgumentException ex) {
            assertThat(ex.getMessage(), equalTo("ERROR: Incorrectly introduced AC"));
        }
    }

    @Test
    @UserStory("US-010")
    public void shouldSelectSeveralStations() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(addStation("Adra"));
        }
    }

    @Test
    @UserStory("US-010")
    public void shouldThrowAnExceptionWhenSelectAnInvalidStation() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(addStation("aStationThatDoesNotExist"));
        } catch (IllegalArgumentException ex) {
            assertThat(ex.getMessage(), equalTo("ERROR: aStationThatDoesNotExist - Incorrectly introduced station"));
        }
    }

    @Test
    @UserStory({"US-012", "US-013"})
    public void shouldSpecifyADate() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(setStartDate(LocalDate.now()));
        }
    }

    @Test
    @UserStory({"US-012", "US-013"})
    public void shouldThrowAnExceptionWhenTryingToSpecifyAnInvalidDate() {
        // TODO
    }

    @Test
    @UserStory("US-017")
    public void shouldRetrieveDataSuccessfullyAfterDoAMeasurementsRequest() throws Exception {
        mockMvc.perform(get("/v1/stations/Arroyo/measurements?startDate=01052018&endDate=01052018"))
                .andExpect(jsonPath("$", hasSize(1)))
                .andExpect(jsonPath("$[0].date", equalTo("2018-05-01")));
    }

    @Test
    @UserStory("US-017")
    public void shouldReturnAnErrorWhenMeasurementsRestResourceInvocationIsWrong() throws Exception {
        mockMvc.perform(get("/v1/stations/Arroyo/measurements?startDate=00052018&endDate=01052018"))
                .andExpect(jsonPath("$.error", equalTo("Invalid provided date (00052018)")));
    }

    @Test
    @UserStory("US-017")
    public void shouldRetrieveDataWithAllParametersAfterDoAMeasurementsRequest() throws Exception {
        mockMvc.perform(get("/v1/stations/Arroyo/measurements?startDate=01052018&endDate=01052018"))
                .andExpect(jsonPath("$", hasSize(1)))
                .andExpect(jsonPath("$[0].averageTemp", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].maxTemp", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].hourMaxTemp", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].minTemp", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].hourMinTemp", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].averageHum", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].maxHum", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].hourMaxHum", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].minHum", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].hourMinHum", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].speedWind", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].directionWind", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].maxSpeedWind", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].hourMaxSpeedWind", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].directionMaxSpeedWind", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].radiation", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].rainfall", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].rainfallEffect", not(isEmptyOrNullString())))
                .andExpect(jsonPath("$[0].eto", not(isEmptyOrNullString())));
    }

    @Test
    @UserStory({"US-029", "US-030"})
    public void shouldRetrieveCorrectlyDataFromVariablesRestResource() throws Exception {
        mockMvc.perform(get("/v1/stations/Arroyo/crops/Arroz/variables?startDate=01052018&endDate=03052018"))
                .andExpect(jsonPath("$", hasSize(3)))
                .andExpect(jsonPath("$[0].date", equalTo("2018-05-01")))
                .andExpect(jsonPath("$[1].date", equalTo("2018-05-02")))
                .andExpect(jsonPath("$[2].date", equalTo("2018-05-03")));
    }

    @Test
    @UserStory({"US-029", "US-030"})
    public void shouldReturnAnErrorWhenVariablesRestResourceInvocationIsWrong() throws Exception {
        mockMvc.perform(get("/v1/stations/Arroyo/crops/Arroz/variables?startDate=WRONG_DATE&endDate=03052018"))
                .andExpect(jsonPath("$.error", equalTo("Invalid provided date (WRONG_DATE)")));
    }

    @Test
    @UserStory({"US-030", "US-031"})
    public void shouldRetrieveLongitudeAndLatitudeFromAnExistingStation() throws Exception {
        mockMvc.perform(get("/v1/stations/Adra"))
                .andExpect(jsonPath("$.utmx", notNullValue()))
                .andExpect(jsonPath("$.utmy", notNullValue()));
    }

    @Test
    @UserStory({"US-030", "US-031"})
    public void shouldReturnAnErrorWhenRequestStationDoesNotExist() throws Exception {
        mockMvc.perform(get("/v1/stations/INVALID_STATION"))
                .andExpect(jsonPath("$.error", equalTo("Station 'INVALID_STATION' not found")));
    }
}
